/* Copyright (C) 2018-2020 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Florian Uhlig [committer] */

#ifndef CBMTRDPARSETASIC_H
#define CBMTRDPARSETASIC_H

#include "CbmTrdParSet.h"  // for CbmTrdParSet

#include <Rtypes.h>      // for THashConsistencyHolder, ClassDef
#include <RtypesCore.h>  // for Int_t, Bool_t, Option_t, UChar_t

#include <map>     // for map
#include <vector>  // for vector

class CbmTrdParAsic;
class FairParamList;

/**
  * \brief Describe TRD module ASIC settings (electronic gain, delays, etc)
  * 
  * The following concepts are used :
  * - DAQ id : unique id of an ASIC composed of the format modId*1000+asicId. 
  * - modId : is calculated according to the CbmTrdAddress::GetModuleId(). 
  * - asicId : uniquely identify the ASICs in one module.
  * - chAddress : identify the read-out pad(s) connected to the ASIC. In the case of FASP it distinguish between R and T pairing    
  **/
class CbmTrdParSetAsic : public CbmTrdParSet {
public:
  /** Standard constructor **/
  CbmTrdParSetAsic(const char* name = "CbmTrdParSetAsic", const char* title = "TRD ASIC parameters",
                   const char* context = "Default");

  /** \brief Destructor **/
  virtual ~CbmTrdParSetAsic();

  /** \brief Reset all parameters **/
  virtual void clear() { ; }
  /** \brief Add ASIC par to the current module
   * It applies to the list of ASICs.
   * \param mod ASIC par (FASP or SPADIC)
   */
  virtual void addParam(CbmTrdParMod* mod);
  /** \brief Add Module list of ASIC par to the setup list
   * It applies to the list of lists.
   * \param module list
   */
  void AddParameters(CbmTrdParSetAsic*);  // (VF) renamed to avoid shadowing of virtual method

  /** \brief Build the ASICs par for the current module from the info stored in the param file 
   * It applies to the list of ASICs.
   * \param module ASICs par (FASP or SPADIC)
   */
  Bool_t getParams(FairParamList*);
  /** \brief Store the ASICs par info for the current module into the param file 
   * It applies to the list of ASICs.
   * \param module ASICs par (FASP or SPADIC)
   */
  void putParams(FairParamList*);

  /** \brief Query the ASICs in the module for their DAQ address. 
   * It applies to the list of ASICs.
   * Returns the list of id of the ASICs within the module.   
   */
  virtual void GetAsicAddresses(std::vector<Int_t>* a) const;
  /** \brief Look for the ASIC which operates on a specific channel
   * It applies to the list of ASICs.
   *\param chAddress Pad address of the channel
   *\return id of the ASIC operating on the channel. -1 in case of failure
   */
  virtual Int_t GetAsicAddress(Int_t chAddress) const;
  /** \brief Look for the ASIC parameters of a given DAQ id
   * It applies to the list of ASICs.
   * \param address ASIC address from DAQ
   * \return A read-only pointer to the parameters 
   */
  virtual const CbmTrdParAsic* GetAsicPar(Int_t address) const { return (CbmTrdParAsic*) GetModulePar(address); }
  /** \brief Look for the ASIC parameters of a given DAQ id
   * It applies to the list of ASICs.
   * \param address ASIC address from DAQ
   * \return A read-write pointer to the parameters 
   */
  virtual CbmTrdParAsic* GetAsicPar(Int_t address) { return (CbmTrdParAsic*) GetModulePar(address); }
  /** \brief Query the type of ASICs in the list
   * It applies to the list of ASICs.
   */
  virtual Int_t GetAsicType() const { return fType; }
  /** \brief Returns the set of ASIC pars corresponding to a whole module
   * It applies to the list of lists.
   * \param detId module Identifier in the CBM geometry
   */
  virtual const CbmTrdParSet* GetModuleSet(Int_t detId) const;
  /** \brief Returns the number of ASICs for the current module
   * It applies to the list of ASICs.
   */
  virtual Int_t GetNofAsics() const { return GetNrOfModules(); }

  virtual void Print(Option_t* opt = "") const;
  /** \brief Initialize the ASIC parameters for DAQ id
   * It applies to the list of ASICs.
   * \param address DAQ id of the ASIC in the current module
   */
  virtual void SetAsicPar(Int_t address, CbmTrdParAsic* p);
  virtual void SetAsicType(Int_t t) { fType = t; }

private:
  CbmTrdParSetAsic(const CbmTrdParSetAsic& ref);
  const CbmTrdParSetAsic& operator=(const CbmTrdParSetAsic& ref);

  UChar_t fType;                               ///< type of ASIC for current se
  std::map<Int_t, CbmTrdParSetAsic*> fModPar;  ///< module wise list of ASIC sets
  ClassDef(CbmTrdParSetAsic,
           1);  // The set of ASIC settings for all TRD modules
};
#endif
